// Generated by delombok at Sun May 29 12:10:02 UTC 2016
package com.github.liblevenshtein.serialization;

import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.net.URI;
import java.net.URL;
import java.nio.file.FileSystem;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.Path;
import lombok.NonNull;

/**
 * Common, serialization routines.
 */
public abstract class AbstractSerializer implements Serializer {
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  private static final org.slf4j.Logger log = org.slf4j.LoggerFactory.getLogger(AbstractSerializer.class);
  /**
   * Operates on filesystem paths.
   */
  @NonNull
  private FileSystem fileSystem = FileSystems.getDefault();

  // Serializers
  // ---------------------------------------------------------------------------
  /**
   * {@inheritDoc}
   */
  @Override
  public void serialize(@NonNull final Serializable object, @NonNull final Path path) throws Exception {
    if (object == null) {
      throw new java.lang.IllegalArgumentException("object is null");
    }
    if (path == null) {
      throw new java.lang.IllegalArgumentException("path is null");
    }
    log.info("Serializing instance of class [{}] to filesystem path [{}]", object.getClass(), path);
    try (OutputStream stream = Files.newOutputStream(path)) {
      serialize(object, stream);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void serialize(@NonNull final Serializable object, @NonNull final File file) throws Exception {
    if (object == null) {
      throw new java.lang.IllegalArgumentException("object is null");
    }
    if (file == null) {
      throw new java.lang.IllegalArgumentException("file is null");
    }
    log.info("Serializing instance of class [{}] to file [{}]", object.getClass(), file);
    serialize(object, file.toPath());
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void serialize(@NonNull final Serializable object, @NonNull final String path) throws Exception {
    if (object == null) {
      throw new java.lang.IllegalArgumentException("object is null");
    }
    if (path == null) {
      throw new java.lang.IllegalArgumentException("path is null");
    }
    log.info("Serializing instance of class [{}] to string path [{}]", object.getClass(), path);
    serialize(object, fileSystem.getPath(path));
  }

  // Deserializers
  // ---------------------------------------------------------------------------
  /**
   * {@inheritDoc}
   */
  @Override
  public <Type extends Serializable> Type deserialize(@NonNull final Class<Type> type, @NonNull final Path path) throws Exception {
    if (type == null) {
      throw new java.lang.IllegalArgumentException("type is null");
    }
    if (path == null) {
      throw new java.lang.IllegalArgumentException("path is null");
    }
    log.info("Deserilizing instance of [{}] from path [{}]", type, path);
    try (InputStream stream = Files.newInputStream(path)) {
      return deserialize(type, stream);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public <Type extends Serializable> Type deserialize(@NonNull final Class<Type> type, @NonNull final File file) throws Exception {
    if (type == null) {
      throw new java.lang.IllegalArgumentException("type is null");
    }
    if (file == null) {
      throw new java.lang.IllegalArgumentException("file is null");
    }
    log.info("Deserilizing instance of [{}] from file [{}]", type, file);
    return deserialize(type, file.toPath());
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public <Type extends Serializable> Type deserialize(@NonNull final Class<Type> type, @NonNull final URL url) throws Exception {
    if (type == null) {
      throw new java.lang.IllegalArgumentException("type is null");
    }
    if (url == null) {
      throw new java.lang.IllegalArgumentException("url is null");
    }
    log.info("Deserilizing instance of [{}] from url [{}]", type, url);
    return deserialize(type, url.openStream());
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public <Type extends Serializable> Type deserialize(@NonNull final Class<Type> type, @NonNull final URI uri) throws Exception {
    if (type == null) {
      throw new java.lang.IllegalArgumentException("type is null");
    }
    if (uri == null) {
      throw new java.lang.IllegalArgumentException("uri is null");
    }
    log.info("Deserilizing instance of [{}] from uri [{}]", type, uri);
    return deserialize(type, uri.toURL());
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public <Type extends Serializable> Type deserialize(@NonNull final Class<Type> type, @NonNull final String pathOrUri) throws Exception {
    if (type == null) {
      throw new java.lang.IllegalArgumentException("type is null");
    }
    if (pathOrUri == null) {
      throw new java.lang.IllegalArgumentException("pathOrUri is null");
    }
    try {
      log.info("Attempting to deserialize instance of [{}] from uri [{}]", type, pathOrUri);
      return deserialize(type, URI.create(pathOrUri));
    } catch (final IllegalArgumentException exception) {
      log.warn("Invalid uri [{}]", pathOrUri);
    }
    log.info("Attempting to deserialize instance of [{}] from path [{}]", type, pathOrUri);
    return deserialize(type, fileSystem.getPath(pathOrUri));
  }

  /**
   * Operates on filesystem paths.
   * @return this
   */
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public AbstractSerializer fileSystem(@NonNull final FileSystem fileSystem) {
    if (fileSystem == null) {
      throw new java.lang.IllegalArgumentException("fileSystem is null");
    }
    this.fileSystem = fileSystem;
    return this;
  }
}
