// Generated by delombok at Sun May 29 12:10:02 UTC 2016
package com.github.liblevenshtein.transducer;

import java.io.Serializable;

/**
 * State along the intersection of the dictionary automaton and the Levenshtein
 * automaton.
 * @param <DictionaryNode> Kind of node in the dictionary automaton.
 * @author Dylon Edwards
 * @since 2.1.0
 */
public final class Intersection<DictionaryNode> implements Serializable {
  private static final long serialVersionUID = 1L;
  /**
   * Intersection along the prefix from the root, dictionary node to the one
   * prior to {@link dictionaryNode}.
   */
  private final Intersection<DictionaryNode> prevIntersection;
  /**
   * Label annotating the edge between the previous dictionary node and
   * {@link #dictionaryNode}.
   */
  private final char label;
  /**
   * Current node in the dictionary, along the intersection's path.
   */
  private final DictionaryNode dictionaryNode;
  /**
   * Current node in the Levenshtein automaton, along the intersection's path.
   */
  private final State levenshteinState;

  /**
   * Constructs an intersection representing the start states of both the
   * dictionary and Levenshtein automata.
   * @param dictionaryRoot Root node of the dictionary automaton.
   * @param initialState Initial state of the Levenshtein automaton.
   */
  public Intersection(final DictionaryNode dictionaryRoot, final State initialState) {
    this(null, '\000', dictionaryRoot, initialState);
  }

  /**
   * Spelling candidate from the dictionary automaton, represented as the prefix
   * of a term in the dictionary constructed by traversing from its root to
   * dictionaryNode, and collecting the characters along the way.
   * @return Spelling candidate from the dictionary automaton.
   */
  public String candidate() {
    return buffer().toString();
  }

  /**
   * Buffers the prefix built by traversion the path from the root node to
   * {@link dictionaryNode}.
   * @return The prefix from the root node to {@link dictionaryNode}.
   */
  private StringBuilder buffer() {
    final StringBuilder buffer;
    if (prevIntersection != null) {
      buffer = prevIntersection.buffer();
      buffer.append(label);
    } else {
      buffer = new StringBuilder();
    }
    return buffer;
  }

  /**
   * Intersection along the prefix from the root, dictionary node to the one
   * prior to {@link dictionaryNode}.
   */
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public Intersection<DictionaryNode> prevIntersection() {
    return this.prevIntersection;
  }

  /**
   * Label annotating the edge between the previous dictionary node and
   * {@link #dictionaryNode}.
   */
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public char label() {
    return this.label;
  }

  /**
   * Current node in the dictionary, along the intersection's path.
   */
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public DictionaryNode dictionaryNode() {
    return this.dictionaryNode;
  }

  /**
   * Current node in the Levenshtein automaton, along the intersection's path.
   */
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public State levenshteinState() {
    return this.levenshteinState;
  }

  @java.lang.Override
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public boolean equals(final java.lang.Object o) {
    if (o == this) return true;
    if (!(o instanceof Intersection)) return false;
    final Intersection<?> other = (Intersection<?>) o;
    final java.lang.Object this$prevIntersection = this.prevIntersection();
    final java.lang.Object other$prevIntersection = other.prevIntersection();
    if (this$prevIntersection == null ? other$prevIntersection != null : !this$prevIntersection.equals(other$prevIntersection)) return false;
    if (this.label() != other.label()) return false;
    final java.lang.Object this$dictionaryNode = this.dictionaryNode();
    final java.lang.Object other$dictionaryNode = other.dictionaryNode();
    if (this$dictionaryNode == null ? other$dictionaryNode != null : !this$dictionaryNode.equals(other$dictionaryNode)) return false;
    final java.lang.Object this$levenshteinState = this.levenshteinState();
    final java.lang.Object other$levenshteinState = other.levenshteinState();
    if (this$levenshteinState == null ? other$levenshteinState != null : !this$levenshteinState.equals(other$levenshteinState)) return false;
    return true;
  }

  @java.lang.Override
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public int hashCode() {
    final int PRIME = 59;
    int result = 1;
    final java.lang.Object $prevIntersection = this.prevIntersection();
    result = result * PRIME + ($prevIntersection == null ? 43 : $prevIntersection.hashCode());
    result = result * PRIME + this.label();
    final java.lang.Object $dictionaryNode = this.dictionaryNode();
    result = result * PRIME + ($dictionaryNode == null ? 43 : $dictionaryNode.hashCode());
    final java.lang.Object $levenshteinState = this.levenshteinState();
    result = result * PRIME + ($levenshteinState == null ? 43 : $levenshteinState.hashCode());
    return result;
  }

  @java.lang.Override
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public java.lang.String toString() {
    return "Intersection(prevIntersection=" + this.prevIntersection() + ", label=" + this.label() + ", dictionaryNode=" + this.dictionaryNode() + ", levenshteinState=" + this.levenshteinState() + ")";
  }

  @java.beans.ConstructorProperties({"prevIntersection", "label", "dictionaryNode", "levenshteinState"})
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public Intersection(final Intersection<DictionaryNode> prevIntersection, final char label, final DictionaryNode dictionaryNode, final State levenshteinState) {
    this.prevIntersection = prevIntersection;
    this.label = label;
    this.dictionaryNode = dictionaryNode;
    this.levenshteinState = levenshteinState;
  }
}
