// Generated by delombok at Sun May 29 12:10:02 UTC 2016
package com.github.liblevenshtein.transducer;

import java.io.Serializable;
import java.util.Comparator;
import com.github.liblevenshtein.transducer.factory.PositionTransitionFactory;
import com.github.liblevenshtein.transducer.factory.StateFactory;

/**
 * Transitions one state to another, given a set of inputs.  This function
 * doesn't need to know about the Levenshtein algorithm, as the
 * algorithm-specific components are injected via setters.
 * @author Dylon Edwards
 * @since 2.1.0
 */
public class StateTransitionFunction implements Serializable {
  private static final long serialVersionUID = 1L;
  /**
   * Sorts {@link State} elements in an unsubsumption-friendly fashion.
   */
  private Comparator<Position> comparator;
  /**
   * Builds and recycles {@link State} instances.
   */
  private StateFactory stateFactory;
  /**
   * Builds position vector, transition functions according to the Levenshtein
   * algorithm.
   */
  private PositionTransitionFactory transitionFactory;
  /**
   * Merges states together according to the Levenshtein algorithm.
   */
  private MergeFunction merge;
  /**
   * Removes positions from a state that are subsumed by other positions in that
   * state.
   */
  private UnsubsumeFunction unsubsume;
  /**
   * Max number of errors tolerated in spelling candidates, from the query term.
   */
  private int maxDistance;
  /**
   * Length of the query term.
   */
  private int queryLength;

  /**
   * Returns the state consisting of all the possible position-transitions from
   * the current state, given the characteristic vector.
   * @param currState Source state to transition from.
   * @param characteristicVector Relevant subwords denoting whether the indices
   * of the current, spelling candidate from the dictionary has characters
   * matching the one being sought from the query term.
   * @return Next state, consisting of all possible position-transitions from
   * the source {@code currState}, conditioned on {@code characteristicVector}.
   */
  public State of(final State currState, final boolean[] characteristicVector) {
    final PositionTransitionFunction transition = transitionFactory.build();
    final int offset = currState.head().termIndex();
    final State nextState = stateFactory.build();
    final int n = maxDistance;
    for (final Position position : currState) {
      final State positions = transition.of(n, position, characteristicVector, offset);
      if (null == positions) {
        continue;
      }
      merge.into(nextState, positions);
    }
    unsubsume.at(nextState, queryLength);
    if (null != nextState.head()) {
      nextState.sort(comparator);
      return nextState;
    }
    return null;
  }

  /**
   * Sorts {@link State} elements in an unsubsumption-friendly fashion.
   * @return this
   */
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public StateTransitionFunction comparator(final Comparator<Position> comparator) {
    this.comparator = comparator;
    return this;
  }

  /**
   * Builds and recycles {@link State} instances.
   * @return this
   */
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public StateTransitionFunction stateFactory(final StateFactory stateFactory) {
    this.stateFactory = stateFactory;
    return this;
  }

  /**
   * Builds position vector, transition functions according to the Levenshtein
   * algorithm.
   * @return this
   */
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public StateTransitionFunction transitionFactory(final PositionTransitionFactory transitionFactory) {
    this.transitionFactory = transitionFactory;
    return this;
  }

  /**
   * Merges states together according to the Levenshtein algorithm.
   * @return this
   */
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public StateTransitionFunction merge(final MergeFunction merge) {
    this.merge = merge;
    return this;
  }

  /**
   * Removes positions from a state that are subsumed by other positions in that
   * state.
   * @return this
   */
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public StateTransitionFunction unsubsume(final UnsubsumeFunction unsubsume) {
    this.unsubsume = unsubsume;
    return this;
  }

  /**
   * Max number of errors tolerated in spelling candidates, from the query term.
   * @return this
   */
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public StateTransitionFunction maxDistance(final int maxDistance) {
    this.maxDistance = maxDistance;
    return this;
  }

  /**
   * Length of the query term.
   * @return this
   */
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public StateTransitionFunction queryLength(final int queryLength) {
    this.queryLength = queryLength;
    return this;
  }
}
