// Generated by delombok at Sun May 29 12:10:02 UTC 2016
package com.github.liblevenshtein.transducer;

import java.io.Serializable;
import lombok.NonNull;
import com.github.liblevenshtein.collection.dictionary.Dawg;
import com.github.liblevenshtein.collection.dictionary.IFinalFunction;
import com.github.liblevenshtein.collection.dictionary.ITransitionFunction;
import com.github.liblevenshtein.transducer.factory.CandidateFactory;
import com.github.liblevenshtein.transducer.factory.StateTransitionFactory;

/**
 * Attributes required for this transducer to search the dictionary.
 * @author Dylon Edwards
 * @since 2.1.2
 * @param <DictionaryNode> Kind of nodes of the dictionary automaton.
 * @param <CandidateType> Kind of the spelling candidates returned from the
 */
public class TransducerAttributes<DictionaryNode, CandidateType> implements Serializable {
  private static final long serialVersionUID = 1L;
  /**
   * Maximum number of spelling errors candidates may have from the query term.
   */
  protected int maxDistance = Integer.MAX_VALUE;
  /**
   * Generates spelling candidates of the requested type. The candidates  which
   * may optionally include the Levenshtein distance between their dictionary
   * terms and the query term.
   */
  @NonNull
  protected CandidateFactory<CandidateType> candidateFactory;
  /**
   * Returns state-transition functions for specific, max edit distances.
   */
  @NonNull
  protected StateTransitionFactory stateTransitionFactory;
  /**
   * Determines the minimum distance at which a Levenshtein state may be
   * considered from the query term, based on its length.
   */
  @NonNull
  protected DistanceFunction minDistance;
  /**
   * Returns whether a dictionary node is the final character in some term.
   */
  @NonNull
  protected IFinalFunction<DictionaryNode> isFinal;
  /**
   * Transition function for dictionary nodes.
   */
  @NonNull
  protected ITransitionFunction<DictionaryNode> dictionaryTransition;
  /**
   * State at which to begin traversing the Levenshtein automaton.
   */
  @NonNull
  protected State initialState;
  /**
   * Root node of the dictionary, at which to begin searching for spelling
   * candidates.
   */
  @NonNull
  protected DictionaryNode dictionaryRoot;
  /**
   * Dictionary of this transducer.
   */
  @NonNull
  protected Dawg dictionary;
  /**
   * Transduction algorithm.
   */
  @NonNull
  protected Algorithm algorithm;
  /**
   * Whether to include the number of errors from the query term with the
   * candidate terms.
   */
  protected boolean includeDistance;

  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  TransducerAttributes(final int maxDistance, @NonNull final CandidateFactory<CandidateType> candidateFactory, @NonNull final StateTransitionFactory stateTransitionFactory, @NonNull final DistanceFunction minDistance, @NonNull final IFinalFunction<DictionaryNode> isFinal, @NonNull final ITransitionFunction<DictionaryNode> dictionaryTransition, @NonNull final State initialState, @NonNull final DictionaryNode dictionaryRoot, @NonNull final Dawg dictionary, @NonNull final Algorithm algorithm, final boolean includeDistance) {
    if (candidateFactory == null) {
      throw new java.lang.IllegalArgumentException("candidateFactory is null");
    }
    if (stateTransitionFactory == null) {
      throw new java.lang.IllegalArgumentException("stateTransitionFactory is null");
    }
    if (minDistance == null) {
      throw new java.lang.IllegalArgumentException("minDistance is null");
    }
    if (isFinal == null) {
      throw new java.lang.IllegalArgumentException("isFinal is null");
    }
    if (dictionaryTransition == null) {
      throw new java.lang.IllegalArgumentException("dictionaryTransition is null");
    }
    if (initialState == null) {
      throw new java.lang.IllegalArgumentException("initialState is null");
    }
    if (dictionaryRoot == null) {
      throw new java.lang.IllegalArgumentException("dictionaryRoot is null");
    }
    if (dictionary == null) {
      throw new java.lang.IllegalArgumentException("dictionary is null");
    }
    if (algorithm == null) {
      throw new java.lang.IllegalArgumentException("algorithm is null");
    }
    this.maxDistance = maxDistance;
    this.candidateFactory = candidateFactory;
    this.stateTransitionFactory = stateTransitionFactory;
    this.minDistance = minDistance;
    this.isFinal = isFinal;
    this.dictionaryTransition = dictionaryTransition;
    this.initialState = initialState;
    this.dictionaryRoot = dictionaryRoot;
    this.dictionary = dictionary;
    this.algorithm = algorithm;
    this.includeDistance = includeDistance;
  }


  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public static class TransducerAttributesBuilder<DictionaryNode, CandidateType> {
    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    private int maxDistance;
    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    private CandidateFactory<CandidateType> candidateFactory;
    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    private StateTransitionFactory stateTransitionFactory;
    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    private DistanceFunction minDistance;
    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    private IFinalFunction<DictionaryNode> isFinal;
    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    private ITransitionFunction<DictionaryNode> dictionaryTransition;
    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    private State initialState;
    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    private DictionaryNode dictionaryRoot;
    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    private Dawg dictionary;
    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    private Algorithm algorithm;
    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    private boolean includeDistance;

    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    TransducerAttributesBuilder() {
    }

    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    public TransducerAttributesBuilder<DictionaryNode, CandidateType> maxDistance(final int maxDistance) {
      this.maxDistance = maxDistance;
      return this;
    }

    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    public TransducerAttributesBuilder<DictionaryNode, CandidateType> candidateFactory(final CandidateFactory<CandidateType> candidateFactory) {
      this.candidateFactory = candidateFactory;
      return this;
    }

    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    public TransducerAttributesBuilder<DictionaryNode, CandidateType> stateTransitionFactory(final StateTransitionFactory stateTransitionFactory) {
      this.stateTransitionFactory = stateTransitionFactory;
      return this;
    }

    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    public TransducerAttributesBuilder<DictionaryNode, CandidateType> minDistance(final DistanceFunction minDistance) {
      this.minDistance = minDistance;
      return this;
    }

    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    public TransducerAttributesBuilder<DictionaryNode, CandidateType> isFinal(final IFinalFunction<DictionaryNode> isFinal) {
      this.isFinal = isFinal;
      return this;
    }

    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    public TransducerAttributesBuilder<DictionaryNode, CandidateType> dictionaryTransition(final ITransitionFunction<DictionaryNode> dictionaryTransition) {
      this.dictionaryTransition = dictionaryTransition;
      return this;
    }

    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    public TransducerAttributesBuilder<DictionaryNode, CandidateType> initialState(final State initialState) {
      this.initialState = initialState;
      return this;
    }

    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    public TransducerAttributesBuilder<DictionaryNode, CandidateType> dictionaryRoot(final DictionaryNode dictionaryRoot) {
      this.dictionaryRoot = dictionaryRoot;
      return this;
    }

    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    public TransducerAttributesBuilder<DictionaryNode, CandidateType> dictionary(final Dawg dictionary) {
      this.dictionary = dictionary;
      return this;
    }

    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    public TransducerAttributesBuilder<DictionaryNode, CandidateType> algorithm(final Algorithm algorithm) {
      this.algorithm = algorithm;
      return this;
    }

    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    public TransducerAttributesBuilder<DictionaryNode, CandidateType> includeDistance(final boolean includeDistance) {
      this.includeDistance = includeDistance;
      return this;
    }

    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    public TransducerAttributes<DictionaryNode, CandidateType> build() {
      return new TransducerAttributes<DictionaryNode, CandidateType>(maxDistance, candidateFactory, stateTransitionFactory, minDistance, isFinal, dictionaryTransition, initialState, dictionaryRoot, dictionary, algorithm, includeDistance);
    }

    @java.lang.Override
    @java.lang.SuppressWarnings("all")
    @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
    @javax.annotation.Generated("lombok")
    public java.lang.String toString() {
      return "TransducerAttributes.TransducerAttributesBuilder(maxDistance=" + this.maxDistance + ", candidateFactory=" + this.candidateFactory + ", stateTransitionFactory=" + this.stateTransitionFactory + ", minDistance=" + this.minDistance + ", isFinal=" + this.isFinal + ", dictionaryTransition=" + this.dictionaryTransition + ", initialState=" + this.initialState + ", dictionaryRoot=" + this.dictionaryRoot + ", dictionary=" + this.dictionary + ", algorithm=" + this.algorithm + ", includeDistance=" + this.includeDistance + ")";
    }
  }

  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public static <DictionaryNode, CandidateType> TransducerAttributesBuilder<DictionaryNode, CandidateType> builder() {
    return new TransducerAttributesBuilder<DictionaryNode, CandidateType>();
  }

  /**
   * Maximum number of spelling errors candidates may have from the query term.
   */
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public int maxDistance() {
    return this.maxDistance;
  }

  /**
   * Generates spelling candidates of the requested type. The candidates  which
   * may optionally include the Levenshtein distance between their dictionary
   * terms and the query term.
   */
  @NonNull
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public CandidateFactory<CandidateType> candidateFactory() {
    return this.candidateFactory;
  }

  /**
   * Returns state-transition functions for specific, max edit distances.
   */
  @NonNull
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public StateTransitionFactory stateTransitionFactory() {
    return this.stateTransitionFactory;
  }

  /**
   * Determines the minimum distance at which a Levenshtein state may be
   * considered from the query term, based on its length.
   */
  @NonNull
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public DistanceFunction minDistance() {
    return this.minDistance;
  }

  /**
   * Returns whether a dictionary node is the final character in some term.
   */
  @NonNull
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public IFinalFunction<DictionaryNode> isFinal() {
    return this.isFinal;
  }

  /**
   * Transition function for dictionary nodes.
   */
  @NonNull
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public ITransitionFunction<DictionaryNode> dictionaryTransition() {
    return this.dictionaryTransition;
  }

  /**
   * State at which to begin traversing the Levenshtein automaton.
   */
  @NonNull
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public State initialState() {
    return this.initialState;
  }

  /**
   * Root node of the dictionary, at which to begin searching for spelling
   * candidates.
   */
  @NonNull
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public DictionaryNode dictionaryRoot() {
    return this.dictionaryRoot;
  }

  /**
   * Dictionary of this transducer.
   */
  @NonNull
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public Dawg dictionary() {
    return this.dictionary;
  }

  /**
   * Transduction algorithm.
   */
  @NonNull
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public Algorithm algorithm() {
    return this.algorithm;
  }

  /**
   * Whether to include the number of errors from the query term with the
   * candidate terms.
   */
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public boolean includeDistance() {
    return this.includeDistance;
  }

  @java.lang.Override
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public java.lang.String toString() {
    return "TransducerAttributes(maxDistance=" + this.maxDistance() + ", dictionary=" + this.dictionary() + ", algorithm=" + this.algorithm() + ", includeDistance=" + this.includeDistance() + ")";
  }

  @java.lang.Override
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public boolean equals(final java.lang.Object o) {
    if (o == this) return true;
    if (!(o instanceof TransducerAttributes)) return false;
    final TransducerAttributes<?, ?> other = (TransducerAttributes<?, ?>) o;
    if (!other.canEqual((java.lang.Object) this)) return false;
    if (this.maxDistance() != other.maxDistance()) return false;
    final java.lang.Object this$dictionary = this.dictionary();
    final java.lang.Object other$dictionary = other.dictionary();
    if (this$dictionary == null ? other$dictionary != null : !this$dictionary.equals(other$dictionary)) return false;
    final java.lang.Object this$algorithm = this.algorithm();
    final java.lang.Object other$algorithm = other.algorithm();
    if (this$algorithm == null ? other$algorithm != null : !this$algorithm.equals(other$algorithm)) return false;
    if (this.includeDistance() != other.includeDistance()) return false;
    return true;
  }

  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  protected boolean canEqual(final java.lang.Object other) {
    return other instanceof TransducerAttributes;
  }

  @java.lang.Override
  @java.lang.SuppressWarnings("all")
  @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
  @javax.annotation.Generated("lombok")
  public int hashCode() {
    final int PRIME = 59;
    int result = 1;
    result = result * PRIME + this.maxDistance();
    final java.lang.Object $dictionary = this.dictionary();
    result = result * PRIME + ($dictionary == null ? 43 : $dictionary.hashCode());
    final java.lang.Object $algorithm = this.algorithm();
    result = result * PRIME + ($algorithm == null ? 43 : $algorithm.hashCode());
    result = result * PRIME + (this.includeDistance() ? 79 : 97);
    return result;
  }
}
